/*********************************************************************************
 ** This file is part of QtContacts tracker storage plugin
 **
 ** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact:  Nokia Corporation (info@qt.nokia.com)
 **
 ** GNU Lesser General Public License Usage
 ** This file may be used under the terms of the GNU Lesser General Public License
 ** version 2.1 as published by the Free Software Foundation and appearing in the
 ** file LICENSE.LGPL included in the packaging of this file.  Please review the
 ** following information to ensure the GNU Lesser General Public License version
 ** 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional rights.
 ** These rights are described in the Nokia Qt LGPL Exception version 1.1, included
 ** in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 *********************************************************************************/

#include <QtCore>

#include <qtcontacts.h>

QTM_USE_NAMESPACE

class Test : public QObject
{
    Q_OBJECT

public:
    Test(QObject *parent = 0)
        : QObject(parent)
    {
        static const QString managerName = QString::fromLatin1("tracker");
        m_manager.reset(new QContactManager(managerName));
        Q_ASSERT(not m_manager.isNull() && m_manager->managerName() == managerName);

        m_request.reset(new QContactFetchRequest());
        m_request->setManager(m_manager.data());
        connect(m_request.data(), SIGNAL(stateChanged(QContactAbstractRequest::State)),
                this, SLOT(stateChanged(QContactAbstractRequest::State)));
    }

    void run()
    {
        m_timer.start();
        m_request->start();
    }

private slots:
    void stateChanged(QContactAbstractRequest::State state)
    {
        switch (state) {
        case QContactAbstractRequest::FinishedState:
        case QContactAbstractRequest::CanceledState:
            qDebug() << "Fetched" << m_request->contacts().size() << "contacts in" << m_timer.elapsed() << "ms";
            QCoreApplication::exit();
        default:
            break;
        }
    }

private:
    QScopedPointer<QContactFetchRequest> m_request;
    QScopedPointer<QContactManager> m_manager;
    QElapsedTimer m_timer;
};

#include "bm_qtcontacts_trackerplugin_fetch.moc"

int
main(int argc, char **argv)
{
    QCoreApplication app(argc, argv);

    // load plugin from build directory
    app.setLibraryPaths(QStringList(QLatin1String("..")) + app.libraryPaths());

    Test instance;
    instance.run();

    return app.exec();
}
