/*********************************************************************************
 ** This file is part of QtContacts tracker storage plugin
 **
 ** Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact:  Nokia Corporation (info@qt.nokia.com)
 **
 ** GNU Lesser General Public License Usage
 ** This file may be used under the terms of the GNU Lesser General Public License
 ** version 2.1 as published by the Free Software Foundation and appearing in the
 ** file LICENSE.LGPL included in the packaging of this file.  Please review the
 ** following information to ensure the GNU Lesser General Public License version
 ** 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional rights.
 ** These rights are described in the Nokia Qt LGPL Exception version 1.1, included
 ** in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 *********************************************************************************/

#ifndef QTRACKERCONTACTFETCHREQUEST_H
#define QTRACKERCONTACTFETCHREQUEST_H

#include "baserequest.h"

#include <cubi.h>

////////////////////////////////////////////////////////////////////////////////////////////////////

class QTrackerContactDetailBuilder;
class QTrackerContactDetail;
class QTrackerContactDetailField;
class QTrackerScalarContactQueryBuilder;

class QSparqlResult;
class QSparqlResultRow;

////////////////////////////////////////////////////////////////////////////////////////////////////

class QTrackerContactFetchRequest : public QTrackerBaseRequest<QContactFetchRequest>
{
    Q_DISABLE_COPY(QTrackerContactFetchRequest)
    Q_OBJECT

    class DetailContext;
    class QueryContext;

    typedef QHash<QContactLocalId, QContact> ContactCache;

public:
    explicit QTrackerContactFetchRequest(QContactAbstractRequest *request,
                                         QContactTrackerEngine *engine,
                                         QObject *parent = 0);
    virtual ~QTrackerContactFetchRequest();

    Cubi::Select query(const QString &contactType, QContactManager::Error &error) const;

protected: // QTrackerAbstractRequest API
    void run();
    void updateRequest(QContactManager::Error error);

private:
    QContactManager::Error bindDetails(QueryContext &context) const;
    QContactManager::Error buildQuery(QueryContext &context) const;

    Cubi::Select baseQuery(const QTrackerScalarContactQueryBuilder &queryBuilder) const;
    void fetchUniqueDetail(QList<QContactDetail> &details,
                           const QueryContext &queryContext,
                           const DetailContext &context,
                           const QString &affiliation);
    void fetchMultiDetails(QList<QContactDetail> &details,
                           const QueryContext &queryContext,
                           const DetailContext &context);
    void fetchMultiDetail(QContactDetail &detail,
                          const DetailContext &context,
                          const QString &rawValueString);
    void fetchResults(ContactCache &results, const QueryContext &context);
    QContactDetail fetchCustomDetail(const QString &rawValue,
                                     const QString &contactType);
    void fetchCustomDetails(const QueryContext &queryContext,
                            ContactCache::Iterator contact);

    QVariant fetchInstances(const QTrackerContactDetailField &field,
                            const QString &rawValueString,
                            bool &isReadOnly) const;
    QVariant fetchField(const QTrackerContactDetailField &field,
                        const QString &rawValueString,
                        bool &isReadOnly) const;
    void fetchCustomValues(const QTrackerContactDetailField &field,
                           QVariant &fieldValue,
                           const QString &rawValueString,
                           bool &isReadOnly) const;
    bool saveDetail(ContactCache::iterator contact, QContactDetail &detail,
                    const QTrackerContactDetail &definition);

    QString fieldStringWithStrippedGraph(const QTrackerContactDetailField &field,
                                         const QString &rawValueString, bool &isOtherGraph) const;
    QStringList fieldStringListWithStrippedGraph(const QTrackerContactDetailField &field,
                                                 const QString &rawValueString, bool &isOtherGraph,
                                                 bool cleanList = false) const;

private: // fields
    const QContactFilter                m_filter;
    const QContactFetchHint             m_fetchHint;
    const QString                       m_nameOrder;
    const QList<QContactSortOrder>      m_sorting;

    QList<QContact>                     m_contacts;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // QTRACKERCONTACTFETCHREQUEST_H
