/*********************************************************************************
 ** This file is part of QtContacts tracker storage plugin
 **
 ** Copyright (c) 2009-2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact:  Nokia Corporation (info@qt.nokia.com)
 **
 ** GNU Lesser General Public License Usage
 ** This file may be used under the terms of the GNU Lesser General Public License
 ** version 2.1 as published by the Free Software Foundation and appearing in the
 ** file LICENSE.LGPL included in the packaging of this file.  Please review the
 ** following information to ensure the GNU Lesser General Public License version
 ** 2.1 requirements will be met:
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional rights.
 ** These rights are described in the Nokia Qt LGPL Exception version 1.1, included
 ** in the file LGPL_EXCEPTION.txt in this package.
 **
 ** Other Usage
 ** Alternatively, this file may be used in accordance with the terms and
 ** conditions contained in a signed written agreement between you and Nokia.
 *********************************************************************************/

#ifndef TRACKERCHANGELISTENER_H_
#define TRACKERCHANGELISTENER_H_

#include <qtcontacts.h>

#include <QtCore/QTimer>
#include <QtSparqlTrackerExtensions/TrackerChangeNotifier>

QTM_USE_NAMESPACE

////////////////////////////////////////////////////////////////////////////////////////////////////

class QContactTrackerEngineParameters;

class QctQueue;
class QctTrackerIdResolver;

////////////////////////////////////////////////////////////////////////////////////////////////////

/*!
 * \class TrackerChangeListener
 * \brief Translates signals from tracker and to contact signals
 * Listen for tracker signals, computes which contacts are and what si changed and emits
 * signals about contact change. Initially designed to compute change signals for
 * QContactTrackerEngine
 */
class QctTrackerChangeListener : public QObject
{
    Q_OBJECT

public:
    explicit QctTrackerChangeListener(const QContactTrackerEngineParameters &params,
                                      QObject *parent = 0);
    virtual ~QctTrackerChangeListener();

signals:
    // signals are with the same semantics as in QContactManagerEngine
    void contactsAdded(const QList<QContactLocalId>& contactIds);
    void contactsChanged(const QList<QContactLocalId>& contactIds);
    void contactsRemoved(const QList<QContactLocalId>& contactIds);
    void relationshipsAdded(const QList<QContactLocalId>& affectedContactIds);
    void relationshipsRemoved(const QList<QContactLocalId>& affectedContactIds);

private slots:
    void onGraphChanged(const QList<TrackerChangeNotifier::Quad>& deletes,
                        const QList<TrackerChangeNotifier::Quad>& inserts);
    void onTrackerIdsResolved();

    void emitQueuedNotifications();

private:
    void processNotifications(QList<TrackerChangeNotifier::Quad> &notifications,
                              QSet<QContactLocalId> &additionsOrRemovals,
                              QSet<QContactLocalId> &relationshipChanges,
                              QSet<QContactLocalId> &propertyChanges);

private:
    QList<TrackerChangeNotifier::Quad> m_deleteNotifications;
    QList<TrackerChangeNotifier::Quad> m_insertNotifications;


    QctTrackerIdResolver *m_resolver;
    QctQueue *m_taskQueue;

    QTimer m_signalCoalescingTimer;

    struct {
        QList<uint> contactClasses;
        uint belongsToGroup;
        uint contactLocalUID;
        uint rdfType;
    } m_trackerIds;

    bool m_debugSignals : 1;
};

#endif /* TRACKERCHANGELISTENER_H_ */
