#!/bin/bash -e

# This file is part of QtContacts tracker storage plugin
#
# Copyright (c) 2010-2011 Nokia Corporation and/or its subsidiary(-ies).
#
# Contact:  Nokia Corporation (info@qt.nokia.com)
#
# GNU Lesser General Public License Usage
# This file may be used under the terms of the GNU Lesser General Public License
# version 2.1 as published by the Free Software Foundation and appearing in the
# file LICENSE.LGPL included in the packaging of this file.  Please review the
# following information to ensure the GNU Lesser General Public License version
# 2.1 requirements will be met:
# http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
#
# In addition, as a special exception, Nokia gives you certain additional rights.
# These rights are described in the Nokia Qt LGPL Exception version 1.1, included
# in the file LGPL_EXCEPTION.txt in this package.
#
# Other Usage
# Alternatively, this file may be used in accordance with the terms and
# conditions contained in a signed written agreement between you and Nokia.

package="libqtcontacts-tracker"
version="unknown version"

topdir=`dirname "$0"`/..
topdir=`cd "$topdir" && pwd`
bindir=/usr/bin

rpath_enabled=`grep ENABLE_BUILDDIRS_RPATH "$topdir/.qmake.cache" | sed -e 's,^.*quote(\([a-z]*\)).*$,\1,'`
pre_steps='tracker-control -rs'
enable_local=no
launcher=
environ=

if ! test "$rpath_enabled" = "yes"
then
    export LD_LIBRARY_PATH=${LD_LIBRARY_PATH:+$LD_LIBRARY_PATH:}$topdir/src/lib
fi

while true
do
    case "$1" in
        -l|-local)
            enable_local=yes
            shift ;;
        -m|-memcheck)
            environ="G_DEBUG=gc-friendly G_SLICE=always-malloc"
            launcher="valgrind --tool=memcheck --leak-check=full --num-callers=50 --suppressions=$topdir/$package.supp "
            shift ;;
        -v|-version)
            version=$2
            shift 2 ;;
        -p|-package)
            package=$2
            shift 2 ;;
        -*)
            echo "$0: Unknown option: $1" >&2
            exit 2 ;;
        *)
            break ;;
    esac
done

cat <<EOF
<!-- generated by $0 on `date` - DO NOT EDIT -->

<testdefinition version="0.1">
  <suite name="$package-tests" timeout="1800">
    <description>Unit and regression tests for $package $version</description>
EOF

for subdir in "$@"
do
    suite=`basename "$subdir"`
    test -f "$subdir/$suite.skip" && continue

    if test "$enable_local" = "yes"
    then
        bindir="$topdir/tests/$suite"
    else
        pre_steps="su -l user -c '$pre_steps'"
        bindir="/usr/bin"
    fi

cat <<EOF
    <set name="$suite">
      <pre_steps>
        <step>$pre_steps</step>
      </pre_steps>

EOF

    "$subdir/$suite" -functions | sed -ne 's/()$//p' | while read test
    do
        attributes="name=\"$test\""
        description=`sed -ne "s|^$suite::$test: ||p" $topdir/EXPECTFAIL || true`

        if test -n "$description"
        then
            attributes="$attributes insignificant=\"true\""
        else
            description="no description available yet"
        fi

        step="${environ}${environ:+ }${launcher}${bindir}/${suite} -maxwarnings 0 ${test}"

        if test "$enable_local" != "yes"
        then
            step="su -l user -c '$step'"
        fi

        cat <<EOF
      <case $attributes>
        <description>$description</description>
        <step>$step</step>
      </case>
EOF
    done

    cat <<EOF
      <environments>
        <scratchbox>true</scratchbox>
        <hardware>true</hardware>
      </environments>
    </set>
EOF

done

cat <<EOF
  </suite>
</testdefinition>
EOF
