<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:template name="caseStats">
    <xsl:param name="allCases" select="true()" />

    <xsl:value-of select="concat(count(.//case[($allCases or @insignificant!='true') and @result='PASS']), ' of ',
                                 count(.//case[($allCases or @insignificant!='true')]), ' passed, ',
                                 count(.//case[($allCases or @insignificant!='true') and @result='FAIL']), ' failed')"/>

    <xsl:if test="count(.//case[($allCases or @insignificant!='true') and @result='FAIL']) > 0">
      <xsl:value-of select="concat(' (', round(count(.//case[($allCases or @insignificant!='true') and @result='FAIL'])
                                           div count(.//case[($allCases or @insignificant!='true')]) * 1000)
                                           div 10, '%)')"/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="/testresults">
    <html>
      <head>
        <title>Test Results - <xsl:value-of select="suite/@name"/></title>
        <style type="text/css"><![CDATA[
          *[onclick] {
            cursor: pointer;
          }

          body {
            color: black;
            background: white;
          }

          dt {
            font-weight: bold;
            margin-right: 1em;
          }

          dl.compact>dt {
            width: 14em;
            float: left;
            clear: left;
          }

          dl.compact>dt:after {
            content: ':';
          }

          dd.stream {
            background: black;
            color: silver;
            clear: left;
            margin-left: 0em;
            white-space: pre;
            padding: 0.5em;
          }

          .insignificant {
            font-style: italic;
          }

          .FAIL {
            color: red;
          }
          .FAIL.insignificant {
            color: silver;
          }
          .PASS.insignificant {
            color: green;
          }

        ]]></style>

        <script type="text/javascript"><![CDATA[
          function toggle(node) {
            if ('none' != node.style.display) {
              node.style.display = 'none';
            } else {
              node.style.display = null;
            }
          }

          function toggleById(id) {
            var elem = document.getElementById(id);

            if (null != elem) {
              toggle(elem);
            }
          }

          function toggleSiblings(node) {
            var tagName = node.tagName;

            while(null != (node = node.nextSibling)) {
              if (node.tagName == tagName) {
                break;
              }

              if (null != node.style) {
                toggle(node);
              }
            }
          }
        ]]></script>
      </head>
      <body>
        <xsl:variable name="significantTestClasses">
          <xsl:if test="count(.//case[@result = 'FAIL' and @insignificant != 'true']) > 0">FAIL</xsl:if>
        </xsl:variable>

        <h1>Test Results - <xsl:value-of select="suite/@name"/></h1>
        <xsl:if test="suite/@description"><p><xsl:copy-of select="suite/@description"/></p></xsl:if>
        <dl class="compact">
          <dt>Environment</dt><dd><xsl:value-of select="@environment"/></dd>
          <dt>Hardware Product</dt><dd><xsl:value-of select="@hwproduct"/></dd>
          <dt>Hardware Build</dt><dd><xsl:value-of select="@hwbuild"/></dd>
          <dt>Full Results (all tests)</dt><dd><xsl:call-template name="caseStats"/></dd>
          <dt>Significant Tests</dt><dd class="{$significantTestClasses}"><xsl:call-template name="caseStats"><xsl:with-param name="allCases" select="false"/></xsl:call-template></dd>
          <dt>Start</dt><dd><xsl:value-of select=".//case[position()=1]/step[position()=1]/start"/></dd>
          <dt>End</dt><dd><xsl:value-of select=".//case[position()=last()]/step[position()=last()]/end"/></dd>
        </dl>

        <xsl:for-each select="suite/set">
          <xsl:variable name="setClasses">
            <xsl:choose>
              <xsl:when test="count(.//case[@result = 'FAIL' and @insignificant != 'true']) > 0">FAIL</xsl:when>
              <xsl:when test="count(.//case[@result = 'FAIL' and @insignificant = 'true']) > 0">FAIL insignificant</xsl:when>
              <xsl:when test="count(.//case[@insignificant='true']) = count(.//case)">insignificant</xsl:when>
            </xsl:choose>
          </xsl:variable>

          <h2 onclick="toggleById('set-{@name}')" class="{$setClasses}"><xsl:value-of select="@name"/></h2>
          <xsl:variable name="style"><xsl:if test="0 = count(.//case[@result='FAIL'])">display: none</xsl:if></xsl:variable>
          <div id="set-{@name}" style="{$style}">
            <dl class="compact">
              <dt>Full Results (all tests)</dt><dd><xsl:call-template name="caseStats"/></dd>
              <dt>Significant Tests</dt><dd><xsl:call-template name="caseStats"><xsl:with-param name="allCases" select="false"/></xsl:call-template></dd>
            </dl>

            <dl class="{@result}">
              <xsl:for-each select="case">
                <xsl:variable name="caseClasses">
                  <xsl:value-of select="@result" />
                  <xsl:if test="'true'=@insignificant"><xsl:text> insignificant</xsl:text></xsl:if>
                </xsl:variable>

                <dt onclick="toggleSiblings(this)" class="{$caseClasses}"><xsl:value-of select="concat(@name, ' - ', @result)"/></dt>
                <dd style="display:none">
                  <dl class="compact">
                    <dt>Description</dt><dd><xsl:value-of select="@description"/></dd>
                    <xsl:for-each select="step">
                      <dt>Start</dt><dd><xsl:value-of select="start/text()"/></dd>
                      <dt>End</dt><dd><xsl:value-of select="end/text()"/></dd>
                      <dt>Output</dt><dd class="stream"><xsl:value-of select="stdout/text()"/></dd>
                      <dt>Errors</dt><dd class="stream"><xsl:value-of select="stderr/text()"/></dd>
                    </xsl:for-each>
                  </dl>
                </dd>
              </xsl:for-each>
            </dl>
          </div>
        </xsl:for-each>
      </body>
    </html>
  </xsl:template>
</xsl:stylesheet>

